#include <gtk/gtk.h>
#include <glib.h>
#include <stdlib.h>
#include <stdio.h>
#include "life.h"
#include "callbacks.h"
#include "interface.h"

#define LOAD	0x1
#define SAVE 	0x2

extern microobType actMicrobType;

extern GdkPixmap *green,*red,*blue,*yellow,*died;
extern GdkBitmap *mask; 
extern GtkWidget *playerMap[MAX_POP][MAX_POP];
extern GtkStyle *style;
extern playerSpecs *actMicrobs,*oldMicrobs,*old2Microbs;
extern gint32 mainLoopInterval;
extern gint runningSimulation;
extern gint infinityChecking;
extern rules rulez;
extern generation *firstGen;
extern gint levelOfGeneration;

gchar *filenameFromSelector;
static int action = NULL;


gint closeDialog (GtkWidget *widget, GdkEvent *event, gpointer data)
{
  gtk_widget_hide (widget);
  gtk_widget_destroy (widget);  
  return TRUE;
}

gint setRulez (GtkWidget *widget, gpointer data)
{
  switch ((gint)data) {
    case 1: rulez.alive_min = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget)); break;
    case 2: rulez.alive_max = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget)); break;
    case 3: rulez.birth_min = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget)); break;
    case 4: rulez.birth_max = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget)); break;
  }
  return TRUE;
}

gint setMainLoopInterval (GtkWidget *widget, gpointer data)
{
  mainLoopInterval = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
  g_print ("setMainLoopInterval: %d",mainLoopInterval);
  return TRUE;
}

gint setInfinity (GtkWidget *widget, gpointer data)
{
  infinityChecking = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(widget));
  g_print ("setInfinity: %d",infinityChecking);
  return TRUE;
}

void storeFileName (GtkWidget *selector, gpointer data)
{
 filenameFromSelector = gtk_file_selection_get_filename (GTK_FILE_SELECTION (selector));
 g_print ("Filename retrieved: [%s]\n", filenameFromSelector);
 if (action == SAVE) saveMap (filenameFromSelector);
 else if (action == LOAD) {
   actMicrobs = loadMap (filenameFromSelector,actMicrobs);
   reDrawPlayerMap (actMicrobs);
 }
 else ;
 action = NULL;
}

void reDrawPlayerMap (playerSpecs *p)
{
int i,ii;
g_print ("reDrawPlayerMap(%p)\n",p);
for (i=0;i<MAX_POP;i++) {
  for (ii=0;ii<MAX_POP;ii++) {
    switch ((*p)[ii][i]) {
	  case mGreen: gtk_pixmap_set (GTK_PIXMAP(GTK_BUTTON(playerMap[ii][i])->child),green,mask); break; 
	  case mRed: gtk_pixmap_set (GTK_PIXMAP(GTK_BUTTON(playerMap[ii][i])->child),red,mask); break;
	  case mBlue: gtk_pixmap_set (GTK_PIXMAP(GTK_BUTTON(playerMap[ii][i])->child),blue,mask); break;
	  case mYellow: gtk_pixmap_set (GTK_PIXMAP(GTK_BUTTON(playerMap[ii][i])->child),yellow,mask); break;
	  case mDied: gtk_pixmap_set (GTK_PIXMAP(GTK_BUTTON(playerMap[ii][i])->child),died,mask); break;
    }

  (*actMicrobs)[ii][i] = (*p)[ii][i];
//  g_print (" Button: [%d][%d]: pressed\n",ii,i);
  }
}
}

void startLifeSimulation(GtkWidget *widget, gpointer data)
{
  runningSimulation = TRUE; 

  if(infinityChecking) 
     copyPlayerSpecs (old2Microbs,oldMicrobs);
   gtk_statusbar_push (GTK_STATUSBAR(mainStatusbar), sbContextId, "Running simulation...");
   runningSimulation = gtk_timeout_add(mainLoopInterval, nextGenWrapper, NULL);
}

void stopLifeSimulation (GtkWidget *widget, gpointer data)
{
   gtk_timeout_remove (runningSimulation);
   gtk_statusbar_push (GTK_STATUSBAR(mainStatusbar), sbContextId, "Ready!");
   runningSimulation = FALSE;
}

void clearTable ( GtkWidget *widget, gpointer data)
{
  int i,ii;

  for (i=0;i<MAX_POP;i++) {
    for (ii=0;ii<MAX_POP;ii++) {
       (*actMicrobs)[ii][i] = mDied;
       gtk_pixmap_set (GTK_PIXMAP(GTK_BUTTON(playerMap[ii][i])->child),died,mask);
    }
  }
  for (i=0;i<levelOfGeneration;i++)
    getLastGeneration(&firstGen);
  levelOfGeneration = 0;
}

void nextGenWrapper (gpointer data) {
    if(infinityChecking) 
      copyPlayerSpecs (old2Microbs,oldMicrobs);
    life (data);
    reDrawPlayerMap (actMicrobs);
}

void nextGenEvent ( GtkWidget *widget, gpointer data)
{
  if (runningSimulation == FALSE) {
    nextGenWrapper (FALSE);
  }
}

void previousGenEvent ( GtkWidget *widget, gpointer data)
{
  playerSpecs *p;
  g_print ("previousEvent()\n");
  if (runningSimulation == FALSE) {
     p = previousGeneration();
     if(p != NULL) {
       copyPlayerSpecs (actMicrobs,p);
       reDrawPlayerMap (actMicrobs);
     }
     else
       gtk_widget_show (createErrorDialog("You are already at the beginnig of life!"));
  }
}

void showStatDialog( GtkWidget *widget, gpointer data)
{
 g_print ("Show status dialog\n");
 gtk_widget_show (createStatDialog());
}

void showRulezDialog( GtkWidget *widget, gpointer data)
{
 g_print ("Show rulez dialog\n");
 gtk_widget_show (createRulezDialog());
}

void showOptionsDialog( GtkWidget *widget, gpointer data)
{
 g_print ("Show options dialog\n");
 gtk_widget_show (createOptionsDialog());
}

void savePlayerMap ( GtkWidget *widget, gpointer data)
{
 GtkWidget *fDialog;
 action = SAVE;
 fDialog = createFileSelectDialog();
 gtk_widget_show (fDialog);
 g_print ("Save dialog \n");
}

void loadPlayerMap ( GtkWidget *widget, gpointer data)
{
 GtkWidget *fDialog;
 action = LOAD;

 fDialog = createFileSelectDialog();
 gtk_widget_show (fDialog);
 g_print ("Load dialog \n");
}

void playerMapClickEvent( GtkWidget *widget, coords *data)
{
  GtkWidget *image;
  
  //if ((*actMicrobs)[data->x][data->y] != actMicrobType) {
    switch (actMicrobType) {
	  case mGreen: gtk_pixmap_set (GTK_PIXMAP(GTK_BUTTON(playerMap[data->x][data->y])->child),green,mask); break; 
	  case mRed: gtk_pixmap_set (GTK_PIXMAP(GTK_BUTTON(playerMap[data->x][data->y])->child),red,mask); break;
	  case mBlue: gtk_pixmap_set (GTK_PIXMAP(GTK_BUTTON(playerMap[data->x][data->y])->child),blue,mask); break;
	  case mYellow: gtk_pixmap_set (GTK_PIXMAP(GTK_BUTTON(playerMap[data->x][data->y])->child),yellow,mask); break;
	  case mDied: gtk_pixmap_set (GTK_PIXMAP(GTK_BUTTON(playerMap[data->x][data->y])->child),died,mask); break;
    }
  //}

  (*actMicrobs)[data->x][data->y] = actMicrobType;
  g_print (" Button: [%d][%d]->[%d]: pressed\n",data->x,data->y,actMicrobType);
}

void setActMicrob(GtkWidget *widget, gpointer data)
{
  gchar buffer[62];
//  sprintf (buffer," : SetActiveMicrob called by : %d", (gint)data);
  gtk_statusbar_push (GTK_STATUSBAR(mainStatusbar), sbContextId, buffer);
  actMicrobType = (gint)data;
}

gint deleteEvent( GtkWidget *widget, GdkEvent *event, gpointer data)
{
  gtk_main_quit ();
  return FALSE;
}
