#include <gtk/gtk.h>
#include <glib.h>
#include <stdlib.h>
#include <stdio.h>
#include "interface.h"
#include "callbacks.h"
#include "life.h"
 
extern microobType actMicrobType;
extern playerSpecs *actMicrobs;
extern gint levelOfGeneration;
extern gint32 mainLoopInterval;
extern gint infinityChecking;
extern rules rulez;

GdkBitmap *mask;
GdkPixmap *green,*red,*blue,*yellow,*died;
GtkStyle *style;

GtkWidget *playerMap[MAX_POP][MAX_POP];

GtkItemFactoryEntry menuItems[] = {
  {"/_File",		NULL,	NULL,	0,	"<Branch>"},
  {"/_File/_New...",	NULL,	clearTable,	0,	NULL},
  {"/_File/sep1",	NULL,	NULL,	0,	"<Separator>"},
  {"/_File/_Save...",	"<alt>S",	savePlayerMap,	0,	NULL},
  {"/_File/_Load...",	"<alt>L",	loadPlayerMap,	0,	NULL},
  {"/_File/sep2",	NULL,	NULL,	0,	"<Separator>"},
  {"/_File/_Quit",	"<alt>Q",	deleteEvent,	0,	NULL},
  {"/_Life",		NULL,	NULL,	0,	"<Branch>"},
  {"/_Life/_Run",	"<alt>R",	startLifeSimulation,	0,	NULL},
  {"/_Life/_Stop",	"<alt>T",	stopLifeSimulation,	0,	NULL},
  {"/_Life/sep3",	NULL,	NULL,	0,	"<Separator>"},
  {"/_Life/_Previows generation",	"<alt>P",	previousGenEvent,	0,	NULL},
  {"/_Life/_Next generation",	"<alt>N",	nextGenEvent,	0,	NULL},
  {"/_Life/sep4",	NULL,	NULL,	0,	"<Separator>"},
  {"/_Life/_Statistics",	"<alt>A",	showStatDialog,	0,	NULL},
  {"/_Microobs",	NULL,	NULL,	0,	"<Branch>"},
  {"/_Microobs/_Red",	"<alt><shift>R",	setActMicrob,	RED_MICROB,	"<RadioItem>"},
  {"/_Microobs/_Blue",	"<alt><shift>B",	setActMicrob,	BLUE_MICROB, "/Microobs/Red"},
  {"/_Microobs/_Yellow",	"<alt><shift>Y",	setActMicrob,	YELLOW_MICROB,	"/Microobs/Red"},
  {"/_Microobs/_Green",	"<alt><shift>G",	setActMicrob,	GREEN_MICROB,	"/Microobs/Red"},
  {"/_Microobs/_Kill",	"<alt><shift>K",	setActMicrob,	DIED_MICROB,	"/Microobs/Red"},
  {"/_Options",		NULL,	NULL,	0,	"<Branch>"},
  {"/_Options/_Configurations...",	NULL,	showOptionsDialog,	0,	NULL},
  {"/_Help",		NULL,	NULL,	0,	"<LastBranch>"},
  {"/_Help/_Rulez...",		NULL,	showRulezDialog,	0,	NULL}
};

void makeMainMenu (GtkWidget *window, GtkWidget **menubar)
{
GtkItemFactory *itemFactory;
GtkAccelGroup *accelGroup;

gint menuItemsSize = sizeof (menuItems) / sizeof (menuItems[0]);

accelGroup = gtk_accel_group_new ();
itemFactory = gtk_item_factory_new (GTK_TYPE_MENU_BAR, "<main>",accelGroup);
gtk_item_factory_create_items (itemFactory, menuItemsSize, menuItems, NULL);
gtk_window_add_accel_group (GTK_WINDOW (window), accelGroup);

if (menubar) 
  *menubar = gtk_item_factory_get_widget (itemFactory,"<main>");
  
}

GtkWidget *getGamePlan (GtkWidget *window) 
{
  gint i = 0, ii = 0;
  GtkWidget *gamePlanH, *gamePlanV;
  GtkPixmap *image;
  coords *cord;

  gamePlanV = gtk_vbox_new (FALSE, 0); 

  for (i = 0;i<20;i++) {
    gamePlanH = gtk_hbox_new (FALSE, 0); 
    gtk_widget_show (gamePlanH);
    for (ii = 0;ii<20;ii++) {
      image = gtk_pixmap_new (died,mask);
      playerMap[ii][i] = gtk_button_new ();
      gtk_container_border_width (GTK_CONTAINER(playerMap[ii][i]),0);
      gtk_widget_set_usize (GTK_WIDGET (playerMap[ii][i]), 25,25);
      gtk_container_add (GTK_CONTAINER (playerMap[ii][i]), image);
      gtk_widget_show (image);
      cord = g_malloc (sizeof (coords));
      cord->x = ii;
      cord->y = i;
      gtk_signal_connect (GTK_OBJECT (playerMap[ii][i]), "clicked", GTK_SIGNAL_FUNC
		      (playerMapClickEvent), (gpointer) cord);
      gtk_widget_show (playerMap[ii][i]);
      gtk_box_pack_start (GTK_BOX (gamePlanH), playerMap[ii][i], FALSE, FALSE, FALSE);
    }
    gtk_box_pack_start (GTK_BOX (gamePlanV), gamePlanH, FALSE, FALSE, FALSE);
    gtk_widget_show (gamePlanV);
  }
return gamePlanV;
}

GtkWidget *createErrorDialog (char *buffer) {
 GtkWidget *window;
 GtkWidget *grid,*frame;
 GtkWidget *label;
 GtkWidget *button;
 
  window = gtk_window_new (GTK_WINDOW_DIALOG);
//  gtk_widget_set_usize (GTK_WIDGET(window),200,160);
  gtk_window_set_title (GTK_WINDOW(window),"Error");
  gtk_signal_connect (GTK_OBJECT(window),"delete-event",GTK_SIGNAL_FUNC(closeDialog), NULL);
  gtk_window_set_position (GTK_WINDOW(window), GTK_WIN_POS_CENTER);
  gtk_window_set_modal (GTK_WINDOW (window), TRUE);
  
  grid = gtk_vbox_new (FALSE, 0);

  frame = gtk_frame_new ("Message");
  label = gtk_label_new (buffer);
  gtk_label_set_justify (GTK_LABEL(label),GTK_JUSTIFY_CENTER);
  gtk_container_add (GTK_CONTAINER(frame),label);
  gtk_widget_show(label);
  gtk_box_pack_start (GTK_BOX(grid),frame,FALSE,FALSE,FALSE);
  gtk_widget_show(frame);

  button = gtk_button_new_with_label("OK");
  gtk_signal_connect_object (GTK_OBJECT(button),"clicked",GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(window));
  gtk_box_pack_start (GTK_BOX(grid),button,FALSE,FALSE,FALSE);
  gtk_widget_show (button);
  gtk_widget_show (grid);

  gtk_container_add (GTK_CONTAINER (window),grid);
  
  return window;
}

GtkWidget *createRulezDialog (void) {
  GtkWidget *window;
  GtkWidget *grid;
  GtkWidget *frame,*button,*label;
  char buffer[100];

  window = gtk_window_new (GTK_WINDOW_DIALOG);
//  gtk_widget_set_usize (GTK_WIDGET(window),200,160);
  gtk_window_set_title (GTK_WINDOW(window),"Rulez");
  gtk_window_set_position (GTK_WINDOW(window), GTK_WIN_POS_CENTER);
  gtk_window_set_modal (GTK_WINDOW (window), TRUE);
  gtk_signal_connect (GTK_OBJECT(window),"delete-event",GTK_SIGNAL_FUNC(closeDialog), NULL);
  
  grid = gtk_vbox_new (FALSE, 0);

  frame = gtk_frame_new ("For alive");
  sprintf (buffer,"If an occopied cell have %d-%d neighbours. \nThen microorganism alive.",rulez.alive_min,rulez.alive_max);
  label = gtk_label_new (buffer);
  gtk_label_set_justify (GTK_LABEL(label),GTK_JUSTIFY_LEFT);
  gtk_container_add (GTK_CONTAINER(frame),label);
  gtk_widget_show(label);
  gtk_box_pack_start (GTK_BOX(grid),frame,FALSE,FALSE,FALSE);
  gtk_widget_show(frame);

  frame = gtk_frame_new ("For birth");
  sprintf (buffer,"If a free cell have %d-%d neighbours. \nThen microorganism will be born there.",rulez.birth_min,rulez.birth_max);
  label = gtk_label_new (buffer);
  gtk_label_set_justify (GTK_LABEL(label),GTK_JUSTIFY_LEFT);
  gtk_container_add (GTK_CONTAINER(frame),label);
  gtk_widget_show(label);
  gtk_box_pack_start (GTK_BOX(grid),frame,FALSE,FALSE,FALSE);
  gtk_widget_show(frame);

  frame = gtk_frame_new ("Else...");
  sprintf (buffer,"The microorganism die.");
  label = gtk_label_new (buffer);
  gtk_label_set_justify (GTK_LABEL(label),GTK_JUSTIFY_LEFT);
  gtk_container_add (GTK_CONTAINER(frame),label);
  gtk_widget_show(label);
  gtk_box_pack_start (GTK_BOX(grid),frame,FALSE,FALSE,FALSE);
  gtk_widget_show(frame);

  button = gtk_button_new_with_label("OK");
  gtk_signal_connect_object (GTK_OBJECT(button),"clicked",GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(window));
  gtk_box_pack_start (GTK_BOX(grid),button,FALSE,FALSE,FALSE);
  gtk_widget_show (button);
  gtk_widget_show (grid);

  gtk_container_add (GTK_CONTAINER (window),grid);
  
  return window;
}

GtkWidget *createMainWindow (GtkWidget *window) {
  
  GtkWidget *menuManagenmentBox, *menuBar;
  GtkWidget *mainGrid;
 

  mainGrid = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (mainGrid);
  mainStatusbar = gtk_statusbar_new();
  sbContextId = gtk_statusbar_get_context_id (GTK_STATUSBAR(mainStatusbar), "Main Status");
  gtk_widget_show(mainStatusbar);
  gtk_statusbar_push (GTK_STATUSBAR(mainStatusbar), sbContextId, "Ready!");

  menuManagenmentBox = gtk_handle_box_new ();
  gtk_widget_show(menuManagenmentBox);


  menuBar = gtk_menu_bar_new ();
  makeMainMenu (window,&menuBar);
  gtk_widget_show (menuBar);
  
  // Assign menubar to menumanager  
  gtk_container_add (GTK_CONTAINER(menuManagenmentBox),menuBar);


  gtk_box_pack_start (GTK_BOX (mainGrid), menuManagenmentBox, FALSE, FALSE, FALSE);
  gtk_box_pack_start (GTK_BOX (mainGrid), getGamePlan(window), FALSE, FALSE, FALSE);
  gtk_box_pack_start (GTK_BOX (mainGrid), mainStatusbar, FALSE, FALSE, FALSE);

  return mainGrid;
}

GtkWidget *createStatDialog(void) 
{
GtkWidget *stat;
GtkWidget *grid;
GtkWidget *button;
GtkWidget *frame,*label;
char buffer[100];

  stat = gtk_window_new (GTK_WINDOW_DIALOG);
//  gtk_widget_set_usize (GTK_WIDGET(stat),200,160);
  gtk_window_set_title (GTK_WINDOW(stat),"Status");
  gtk_window_set_position (GTK_WINDOW(stat), GTK_WIN_POS_CENTER);
  gtk_window_set_modal (GTK_WINDOW (stat), TRUE);
  gtk_signal_connect (GTK_OBJECT(stat),"delete-event",GTK_SIGNAL_FUNC(closeDialog), NULL);
  
  grid = gtk_vbox_new (FALSE, 0);

  frame = gtk_frame_new ("Count of each micobs");
  sprintf (buffer,"Red: %d\nBlue: %d\nGreen: %d\nYellow: %d\n\nLevel: %d",mCount (mRed,actMicrobs), mCount (mBlue,actMicrobs), mCount (mGreen,actMicrobs), mCount (mYellow,actMicrobs), levelOfGeneration);
  label = gtk_label_new (buffer);
  gtk_label_set_justify (GTK_LABEL(label),GTK_JUSTIFY_LEFT);
  gtk_container_add (GTK_CONTAINER(frame),label);
  gtk_widget_show(label);
  gtk_box_pack_start (GTK_BOX(grid),frame,FALSE,FALSE,FALSE);
  gtk_widget_show(frame);

  button = gtk_button_new_with_label("OK");
  gtk_signal_connect_object (GTK_OBJECT(button),"clicked",GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(stat));
  gtk_box_pack_start (GTK_BOX(grid),button,FALSE,FALSE,FALSE);
  gtk_widget_show (button);
  gtk_widget_show (grid);

  gtk_container_add (GTK_CONTAINER (stat),grid);
  
  return stat;
}

GtkWidget *createFileSelectDialog (void)
{
  GtkWidget *fDialog;
//  GtkWidget *okButton;
//  GtkWidget *cancelButton;

  fDialog = gtk_file_selection_new ("Select file...");
  GTK_WINDOW (fDialog)->type = GTK_WINDOW_DIALOG;
  gtk_window_set_position (GTK_WINDOW(fDialog),GTK_WIN_POS_CENTER);
  gtk_window_set_modal (GTK_WINDOW(fDialog),TRUE);
  gtk_widget_show (fDialog);

  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION(fDialog)->ok_button),"clicked",GTK_SIGNAL_FUNC(storeFileName),(gpointer) fDialog);
  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION(fDialog)->ok_button), "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy), (gpointer) fDialog);
  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION(fDialog)->cancel_button), "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy), (gpointer) fDialog);
/*
  okButton = GTK_FILE_SELECTION (fDialog)->ok_button;
  gtk_object_set_data (GTK_OBJECT (fDialog),"okButton",okButton);
  gtk_widget_show (okButton);
  GTK_WIDGET_SET_FLAGS (okButton,GTK_CAN_DEFAULT);

  cancelButton = GTK_FILE_SELECTION (fDialog)->cancel_button;
  gtk_object_set_data (GTK_OBJECT (fDialog),"okButton",cancelButton);
  gtk_widget_show (cancelButton);
  GTK_WIDGET_SET_FLAGS (cancelButton,GTK_CAN_DEFAULT);
*/
  return fDialog;
}


void initializatePixmaps (GtkWidget *window)
{
  style = gtk_widget_get_style (window);

  died = gdk_pixmap_create_from_xpm (window->window, &mask, &style->bg[GTK_STATE_NORMAL], "pixmaps/died.xpm");
  blue = gdk_pixmap_create_from_xpm (window->window, &mask, &style->bg[GTK_STATE_NORMAL], "pixmaps/blue.xpm");
  red = gdk_pixmap_create_from_xpm (window->window, &mask, &style->bg[GTK_STATE_NORMAL], "pixmaps/red.xpm");
  green = gdk_pixmap_create_from_xpm (window->window, &mask, &style->bg[GTK_STATE_NORMAL], "pixmaps/green.xpm");
  yellow = gdk_pixmap_create_from_xpm (window->window, &mask, &style->bg[GTK_STATE_NORMAL], "pixmaps/yellow.xpm");
}

GtkWidget*
createOptionsDialog (void)
{
  GtkWidget *window;
  GtkWidget *vbox1;
  GtkWidget *frame1;
  GtkWidget *vbox2;
  GtkWidget *hbox1;
  GtkWidget *label1;
  GtkObject *spinbutton1_adj;
  GtkWidget *spinbutton1;
  GtkWidget *label2;
  GtkObject *spinbutton2_adj;
  GtkWidget *spinbutton2;
  GtkWidget *hbox2;
  GtkWidget *label3;
  GtkObject *spinbutton3_adj;
  GtkWidget *spinbutton3;
  GtkWidget *label4;
  GtkObject *spinbutton4_adj;
  GtkWidget *spinbutton4;
  GtkWidget *interval;
  GtkWidget *hbox3;
  GtkWidget *label5;
  GtkObject *spinbutton5_adj;
  GtkWidget *spinbutton5;
  GtkWidget *label6;
  GtkWidget *frame3;
  GtkWidget *checkbutton1;
  GtkWidget *button1;

  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_object_set_data (GTK_OBJECT (window), "window", window);
  gtk_window_set_title (GTK_WINDOW (window), "Options");
  gtk_window_set_position (GTK_WINDOW(window), GTK_WIN_POS_CENTER);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox1);
  gtk_object_set_data_full (GTK_OBJECT (window), "vbox1", vbox1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (window), vbox1);

  frame1 = gtk_frame_new ("Rulez");
  gtk_widget_ref (frame1);
  gtk_object_set_data_full (GTK_OBJECT (window), "frame1", frame1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame1);
  gtk_box_pack_start (GTK_BOX (vbox1), frame1, TRUE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (frame1), 5);

  vbox2 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox2);
  gtk_object_set_data_full (GTK_OBJECT (window), "vbox2", vbox2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox2);
  gtk_container_add (GTK_CONTAINER (frame1), vbox2);

  hbox1 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox1);
  gtk_object_set_data_full (GTK_OBJECT (window), "hbox1", hbox1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox1);
  gtk_box_pack_start (GTK_BOX (vbox2), hbox1, TRUE, TRUE, 0);

  label1 = gtk_label_new ("For alive: ");
  gtk_widget_ref (label1);
  gtk_object_set_data_full (GTK_OBJECT (window), "label1", label1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label1);
  gtk_box_pack_start (GTK_BOX (hbox1), label1, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label1), GTK_JUSTIFY_RIGHT);

  spinbutton1_adj = gtk_adjustment_new (rulez.alive_min, 1, 7, 1, 10, 10);
  spinbutton1 = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton1_adj), 1, 0);
  gtk_widget_ref (spinbutton1);
  gtk_object_set_data_full (GTK_OBJECT (window), "spinbutton1", spinbutton1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (spinbutton1);
  gtk_box_pack_start (GTK_BOX (hbox1), spinbutton1, TRUE, TRUE, 0);

  label2 = gtk_label_new (" - ");
  gtk_widget_ref (label2);
  gtk_object_set_data_full (GTK_OBJECT (window), "label2", label2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label2);
  gtk_box_pack_start (GTK_BOX (hbox1), label2, FALSE, FALSE, 0);

  spinbutton2_adj = gtk_adjustment_new (rulez.alive_max, 1, 7, 1, 10, 10);
  spinbutton2 = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton2_adj), 1, 0);
  gtk_widget_ref (spinbutton2);
  gtk_object_set_data_full (GTK_OBJECT (window), "spinbutton2", spinbutton2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (spinbutton2);
  gtk_box_pack_start (GTK_BOX (hbox1), spinbutton2, TRUE, TRUE, 0);

  hbox2 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox2);
  gtk_object_set_data_full (GTK_OBJECT (window), "hbox2", hbox2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox2);
  gtk_box_pack_start (GTK_BOX (vbox2), hbox2, FALSE, FALSE, 0);

  label3 = gtk_label_new ("For Birth: ");
  gtk_widget_ref (label3);
  gtk_object_set_data_full (GTK_OBJECT (window), "label3", label3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label3);
  gtk_box_pack_start (GTK_BOX (hbox2), label3, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label3), GTK_JUSTIFY_RIGHT);

  spinbutton3_adj = gtk_adjustment_new (rulez.birth_min, 1, 7, 1, 10, 10);
  spinbutton3 = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton3_adj), 1, 0);
  gtk_widget_ref (spinbutton3);
  gtk_object_set_data_full (GTK_OBJECT (window), "spinbutton3", spinbutton3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (spinbutton3);
  gtk_box_pack_start (GTK_BOX (hbox2), spinbutton3, TRUE, TRUE, 0);

  label4 = gtk_label_new (" - ");
  gtk_widget_ref (label4);
  gtk_object_set_data_full (GTK_OBJECT (window), "label4", label4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label4);
  gtk_box_pack_start (GTK_BOX (hbox2), label4, FALSE, FALSE, 0);

  spinbutton4_adj = gtk_adjustment_new (rulez.birth_max, 1, 7, 1, 10, 10);
  spinbutton4 = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton4_adj), 1, 0);
  gtk_widget_ref (spinbutton4);
  gtk_object_set_data_full (GTK_OBJECT (window), "spinbutton4", spinbutton4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (spinbutton4);
  gtk_box_pack_start (GTK_BOX (hbox2), spinbutton4, TRUE, TRUE, 0);

  interval = gtk_frame_new ("Interval");
  gtk_widget_ref (interval);
  gtk_object_set_data_full (GTK_OBJECT (window), "interval", interval,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (interval);
  gtk_box_pack_start (GTK_BOX (vbox1), interval, TRUE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (interval), 5);

  hbox3 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox3);
  gtk_object_set_data_full (GTK_OBJECT (window), "hbox3", hbox3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox3);
  gtk_container_add (GTK_CONTAINER (interval), hbox3);

  label5 = gtk_label_new ("mainLoopInterval: ");
  gtk_widget_ref (label5);
  gtk_object_set_data_full (GTK_OBJECT (window), "label5", label5,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label5);
  gtk_box_pack_start (GTK_BOX (hbox3), label5, FALSE, FALSE, 0);

  spinbutton5_adj = gtk_adjustment_new (100, 1, 1000, 10, 10, 10);
  spinbutton5 = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton5_adj), 1, 0);
  gtk_widget_ref (spinbutton5);
  gtk_object_set_data_full (GTK_OBJECT (window), "spinbutton5", spinbutton5,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (spinbutton5);
  gtk_box_pack_start (GTK_BOX (hbox3), spinbutton5, TRUE, TRUE, 0);

  label6 = gtk_label_new (" ms");
  gtk_widget_ref (label6);
  gtk_object_set_data_full (GTK_OBJECT (window), "label6", label6,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label6);
  gtk_box_pack_start (GTK_BOX (hbox3), label6, FALSE, FALSE, 0);

  frame3 = gtk_frame_new ("Options");
  gtk_widget_ref (frame3);
  gtk_object_set_data_full (GTK_OBJECT (window), "frame3", frame3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame3);
  gtk_box_pack_start (GTK_BOX (vbox1), frame3, TRUE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (frame3), 5);

  checkbutton1 = gtk_check_button_new_with_label ("Check for infinity loops");
  gtk_widget_ref (checkbutton1);
  gtk_object_set_data_full (GTK_OBJECT (window), "checkbutton1", checkbutton1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (checkbutton1);
  gtk_container_add (GTK_CONTAINER (frame3), checkbutton1);
  gtk_container_set_border_width (GTK_CONTAINER (checkbutton1), 5);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton1), TRUE);

  button1 = gtk_button_new_with_label ("OK");
  gtk_widget_ref (button1);
  gtk_object_set_data_full (GTK_OBJECT (window), "button1", button1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button1);
  gtk_box_pack_start (GTK_BOX (vbox1), button1, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (button1), 5);

  gtk_signal_connect (GTK_OBJECT (window), "delete-event",
                      GTK_SIGNAL_FUNC (closeDialog),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (spinbutton1), "changed",
                      GTK_SIGNAL_FUNC (setRulez),
                      (gpointer)1);
  gtk_signal_connect (GTK_OBJECT (spinbutton2), "changed",
                      GTK_SIGNAL_FUNC (setRulez),
                      (gpointer)2);
  gtk_signal_connect (GTK_OBJECT (spinbutton3), "changed",
                      GTK_SIGNAL_FUNC (setRulez),
                      (gpointer)3);
  gtk_signal_connect (GTK_OBJECT (spinbutton4), "changed",
                      GTK_SIGNAL_FUNC (setRulez),
                      (gpointer)4);
  gtk_signal_connect (GTK_OBJECT (spinbutton5), "changed",
                      GTK_SIGNAL_FUNC (setMainLoopInterval),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (checkbutton1), "toggled",
                      GTK_SIGNAL_FUNC (setInfinity),
                      NULL);
  gtk_signal_connect_object (GTK_OBJECT (button1), "clicked",
                             GTK_SIGNAL_FUNC (gtk_widget_destroy),
                             GTK_OBJECT (window));

  return window;
}

